use std::path::Path;

use crate::Component;

use super::cuddle_vars::{load_cuddle_file, CuddleVariable, CuddleVariables};

pub enum IngressType {
    External,
    Internal,
}

pub struct Ingress {
    variables: CuddleVariables,
}

impl Ingress {
    pub async fn new(path: &Path) -> anyhow::Result<Self> {
        let variables = load_cuddle_file(path).await?;

        Ok(Self { variables })
    }
}

impl Component for Ingress {
    fn name(&self) -> String {
        "cuddle/ingress".to_string()
    }

    fn render(
        &self,
        _environment: &str,
        _value: &serde_yaml::Value,
    ) -> Option<anyhow::Result<(String, String)>> {
        if let Some(ingress_types) = self
            .variables
            .0
            .get("ingress")
            .and_then(|v| match v {
                CuddleVariable::Array(a) => Some(a),
                _ => None,
            })
            .map(|o| {
                let mut types = Vec::new();

                for value in o {
                    match value {
                        CuddleVariable::Object(o) => {
                            if o.0.contains_key("external") {
                                types.push(IngressType::External)
                            } else if o.0.contains_key("internal") {
                                types.push(IngressType::Internal)
                            } else {
                                continue;
                            }
                        }
                        _ => continue,
                    }
                }

                types
            })
        {
            let mut templates = Vec::new();
            for ingress_type in ingress_types {
                match ingress_type {
                    IngressType::External => templates.push(
                        r#"
{%- set service_name = vars.cuddle_vars.service %}
{%- set host_name = vars.cuddle_vars.service | replace("_", "-") | replace(".", "-")  %}
---
apiVersion: networking.k8s.io/v1
kind: Ingress
metadata:
  annotations:
    cert-manager.io/issuer: kjuulh-app
    traefik.ingress.kubernetes.io/router.entrypoints: web
    traefik.ingress.kubernetes.io/router.tls: "true"
  labels:
    app: {{ service_name }}
    cluster: {{ vars.cluster_vars.namespace }}
  name: {{ service_name }}-external-http
  namespace: {{ vars.cluster_vars.namespace }}
spec:
  rules:
  - host: {{ host_name }}.{{ environment }}.kjuulh.app
    http:
      paths:
      - backend:
          service:
            name: {{ service_name }}
            port:
              name: external-http
        path: /
        pathType: Prefix
  tls:
  - hosts:
    - {{ host_name }}.{{ environment }}.kjuulh.app
    secretName: tls-{{ service_name }}-kjuulh-app-external-ingress-dns
"#,
                    ),
                    IngressType::Internal => templates.push(
                        r#"
{%- set service_name = vars.cuddle_vars.service %}
{%- set host_name = vars.cuddle_vars.service | replace("_", "-") | replace(".", "-")  %}
---
apiVersion: networking.k8s.io/v1
kind: Ingress
metadata:
  annotations:
    cert-manager.io/issuer: kjuulh-app
    traefik.ingress.kubernetes.io/router.entrypoints: web
    traefik.ingress.kubernetes.io/router.tls: "true"
  labels:
    app: {{ service_name }}
    cluster: {{ vars.cluster_vars.namespace }}
  name: {{ service_name }}-internal-http
  namespace: {{ vars.cluster_vars.namespace }}
spec:
  rules:
  - host: {{ host_name }}.{{ environment }}.internal.kjuulh.app
    http:
      paths:
      - backend:
          service:
            name: {{ service_name }}
            port:
              name: internal-http
        path: /
        pathType: Prefix
  tls:
  - hosts:
    - {{ host_name }}.{{ environment }}.internal.kjuulh.app
    secretName: tls-{{ service_name }}-kjuulh-app-internal-ingress-dns
"#,
                    ),
                }
            }

            return Some(Ok(("ingress.yaml".into(), templates.join("\n"))));
        }

        None
    }
}
